<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Tco01800FatorCorrecao extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $novoCodigo;
    protected $Tco01800FatorCorrecaoservice;

    public function __construct($crudService, $parameterBag, $environment, $entityManager, $dataSourceManager, $novoCodigo) {
        $this->crudService                = $crudService;
        $this->environment                = $environment;
        $this->entityManager              = $entityManager;
        $this->dataSourceManager          = $dataSourceManager;
        $this->novoCodigo                 = $novoCodigo;
    }
    
    public function salvaNovaTabFatorCorrecao(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        if($row['__is_new']){
            $row['NRTABCORRPRODTCO'] = $this->novoCodigo->getCodeByTableName("tabcorrprodtco", $this->environment->getNrOrgTrab());
        }
        
        $row['VRDEGELO'] = $this->valueToFloat($row['VRDEGELO']);
        $row['VRLIMPEZA'] = $this->valueToFloat($row['VRLIMPEZA']);
        $row['VRCOCCAO'] = $this->valueToFloat($row['VRCOCCAO']);
        
        $tabFatorCorrecaoExistente = $this->fatorCorrecaoExistente($row);
        
        if($tabFatorCorrecaoExistente) {
            $errorData = array( array( "ERRORMSG" => "TCO01800_FATO_CORRECAO_EXISTENTE",
                                       "NMGRUPOPRATO" => $tabFatorCorrecaoExistente['NMGRUPOPRATO'],
                                       "NMTIPOPREPAROTCO" => $tabFatorCorrecaoExistente['NMTIPOPREPAROTCO']));
            $response->addDataSet(new DataSet ('tabfatorcorrecaoexistente', $errorData));
        } else {
            $rows = $this->crudService->getRowsToSave($request, "tabcorrprodtco");
            $this->crudService->save("tabcorrprodtco", $rows);
            $response->addDataSet(new DataSet ('tabcorrprodtco', $rows));
        }
        
    }
    
    public function salvaEdicaoTabFatorCorrecao(DTO\Request $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        
        foreach($rows as $row) {
            $row['VRDEGELO'] = $this->valueToFloat($row['VRDEGELO']);
            $row['VRLIMPEZA'] = $this->valueToFloat($row['VRLIMPEZA']);
            $row['VRCOCCAO'] = $this->valueToFloat($row['VRCOCCAO']);
        }
        
        $rowsToSave = $this->crudService->getRowsToSave($request, "tabcorrprodtco");
        $this->crudService->save("tabcorrprodtco", $rowsToSave);
        $response->addDataSet(new DataSet ('tabcorrprodtco', $rowsToSave));
    }
    
    private function valueToFloat($value) {
        $newValue = str_replace(".", "", $value);
        $newValue = str_replace(",", ".", $newValue);
        return floatval($newValue);
    }
    
    public function excluiTabFatorCorrecao(DTO\Request $request, DTO\Response $response) {
        $rows = $this->crudService->getRowsToDelete($request, 'tabcorrprodtco');
        $dataSet = new DataSet("tabcorrprodtco", $rows);
		$this->crudService->delete($response, $dataSet);
    }
    
    public function fatorCorrecaoExistente($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT TABCORRPRODTCO.NRTABCORRPRODTCO, 
                       TABCORRPRODTCO.NRGRUPOPRATO, 
                       GRUPOPRATO.CDGRUPOPRATO, 
                       GRUPOPRATO.NMGRUPOPRATO, 
                       TABCORRPRODTCO.NRTIPOPREPAROTCO, 
                       TIPOPREPAROTCO.NMTIPOPREPAROTCO, 
                       TABCORRPRODTCO.CDPRODUTO, 
                       TABCORRPRODTCO.VRDEGELO, 
                       TABCORRPRODTCO.VRLIMPEZA, 
                       TABCORRPRODTCO.VRCOCCAO, 
                       TABCORRPRODTCO.NRORG, 
                       TABCORRPRODTCO.IDATIVO 
                  FROM TABCORRPRODTCO 
                 INNER JOIN GRUPOPRATO 
                    ON GRUPOPRATO.NRGRUPOPRATO = TABCORRPRODTCO.NRGRUPOPRATO 
                   AND GRUPOPRATO.CDFILIAL = ( SELECT PARAMTCO.CDFILIAL 
                                                 FROM PARAMTCO 
                                                WHERE PARAMTCO.NRORG = :NRORG ) 
                 INNER JOIN TIPOPREPAROTCO 
                    ON TIPOPREPAROTCO.NRTIPOPREPAROTCO = TABCORRPRODTCO.NRTIPOPREPAROTCO 
                 WHERE TABCORRPRODTCO.NRGRUPOPRATO = :NRGRUPOPRATO
                   AND (TABCORRPRODTCO.CDPRODUTO = :CDPRODUTO OR (:CDPRODUTO IS NULL AND TABCORRPRODTCO.CDPRODUTO IS NULL))
                   AND TABCORRPRODTCO.NRTIPOPREPAROTCO = :NRTIPOPREPAROTCO";

        $params = array(
            "NRORG"            => $this->environment->getNrOrgTrab(),
            "NRGRUPOPRATO"     => $row['NRGRUPOPRATO'],
            "CDPRODUTO"        => $row['CDPRODUTO'] != "" && $row['CDPRODUTO'] != null? $row['CDPRODUTO']: null,
            "NRTIPOPREPAROTCO" => $row['NRTIPOPREPAROTCO']
        );

        $query = $connection->prepare($sql);
        $query->execute($params);

        return $query->fetch(); 
    }
    
}